// ================================================================
// Commaize a number.
// ================================================================
#include <cstdlib>
#include <string>
#include <iostream>
#include <iomanip>
using namespace std;

template <typename T> string commaize_tostring(T in) {return to_string(in);}
string commaize_tostring(const string& in) {return in;}

/**
 * Insert thousands separator into a number and, optionally, specify
 * the number of decimal places.
 * 
 * Here are some examples that demonstrate how it is used.
 * 
 *   string x;
 *   x=commaize(1234)                            # 1,234
 *   x=commaize(-1234)                           # -1,234
 *   x=commaize(12345.678)                       # 12,345.678
 *   x=commaize(-12345.678)                      # -12,345.678
 *   x=commaize(12345.678, dpl=2)                # 12,345.69
 *   x=commaize(12345.678, dpl=1)                # 12,345.7
 *   x=commaize(1234,sep='.')                    # 1.234
 *   x=commaize("1234567,89", sep='.', dpt=',')  # EU: 1.234.567,89
 * 
 * The arguments are:
 * 
 * @param num  The number: int, float or string.
 * @param dpl  Decimal places (default: -1 -> do nothing).
 * @param sep  Thousands separator (default: ",").
 * @param dpt  Decimal point (default: ".").
 * @returns a formatted string.
 */
template <typename T>
string commaize(T num, int dpl=-1, char sep=',', char dpt='.')
{
  string characteristic = commaize_tostring(num);
  if (dpl >= 0) {
    // Use sprintf() to do the rounding.
    double val = atof(characteristic.c_str());
    char fmt[128];
    char tmp[128];
    sprintf(fmt, "%%.%df", dpl);
    sprintf(tmp, fmt, val);
    characteristic = tmp;
  }

  string neg="";
  if (characteristic[0] == '-') {
    neg = "-";
    characteristic = characteristic.substr(1, string::npos);
  }

  string mantissa = "";
  size_t pos = characteristic.find(dpt);
  if (pos != string::npos) {
    // There is a decimal point, break out
    // the characteristic and the mantissa
    // because we only care about the
    // characteristic for commas. We will
    // append the mantissa at the end.
    mantissa = characteristic.substr(pos+1);

    // Trim trailing zeros.
    if (dpl < 0) {
      while (mantissa.size() > 0 && mantissa[mantissa.size()-1] == '0') {
        mantissa = mantissa.substr(0, mantissa.size() - 1);
      }
    }
    characteristic = characteristic.substr(0, pos);
  }

  // Insert the commas.
  size_t j = characteristic.size() % 3;
  string result = "";
  result.reserve(characteristic.size() + ((characteristic.size()/3) + 2 + mantissa.size()));
  for(size_t i=0; i<characteristic.size(); i++) {
    if ((i%3) == j && i > 0) {
      result += sep;
    }
    result += characteristic[i];
  }
  if (!mantissa.empty()) {
    char dpts[2] = {dpt, 0};
    result += string(dpts) + mantissa;
  }
  return neg + result;
}

/**
 * main
 */
int main()
{
  string result;
  string str;
  unsigned id = 0;

  // Test positive integers.
  size_t nums[] = {1, 12, 123, 1234, 12345, 123456, 1234567, 12345678, 123456789};
  for(auto num : nums) {
    result = commaize(num);
    cout  << setw(4) << right << ++id << " "
          << right << setw(20) << num
          << "   "
          << right << setw(20) << result
          << endl;
  }

  // Test positive doubles.
  double dnums[] = {1.23, 12.34, 123.45, 1234.56, 12345.67, 123456.78, 1234567.89, 12345678.9, 1.0};
  for(auto dnum : dnums) {
    result = commaize(dnum);
    cout  << setw(4) << right << ++id << " "
          << right << fixed << setw(20) << dnum
          << "   "
          << right << setw(20) << result
          << endl;
  }

  // Test negative integers.
  for(long num: nums) {
    num = -num;
    result = commaize(num);
    cout  << setw(4) << right << ++id << " "
          << right << setw(20) << num
          << "   "
          << right << setw(20) << result
          << endl;
  }

  // Test negative doubles.
  for(auto num: dnums) {
    num = -num;
    result = commaize(num);
    cout  << setw(4) << right << ++id << " "
          << right << fixed << setprecision(2) << setw(20) << num
          << "   "
          << right << setw(20) << result
          << endl;
  }

  // String test.
  str = "123456789";
  result = commaize(str);
  cout  << setw(4) << right << ++id << " "
        << right << setw(20) << str
        << "   "
        << right << setw(20) << result
        << endl;

  // EU style
  str = "12345678,9";
  result = commaize(str, -1, '.', ',');
  cout  << setw(4) << right << ++id << " "
        << right << fixed << setprecision(2) << setw(20) << str
        << "   "
        << right << setw(20) << result
        << "  # EU style"
        << endl;

  // dollarize
  // Round to 2 significant digits.
  double ddata[] = {12345.123, 12345, 12345.1251};
  for(auto datum : ddata) {
    result = commaize(datum, 2);
    cout  << setw(4) << right << ++id << " "
          << right << fixed << setw(20) << datum
          << "   "
          << right << setw(20) << result
          << "  # 2 decimal places"
          << endl;
  }

  // 1 decimal place.
  string datum = "1234567.89";
  result = commaize(datum, 1);
  cout  << setw(4) << right << ++id << " "
        << right << fixed << setw(20) << datum
        << "   "
        << right << setw(20) << result
        << "  # 1 decimal place"
        << endl;
}
