<?php
#
# Example that shows how to insert commas into a number.
#

# ================================================================
# Insert thousands separator into a number and, optionally, specify
# the number of decimal places.
#    
# Here are some examples that demonstrate how it is used.
#    
#   x=commaize(1234)                        # 1,234
#   x=commaize(-1234)                       # -1,234
#   x=commaize(12345.678)                   # 12,345.678
#   x=commaize(-12345.678)                  # -12,345.678
#   x=commaize(12345.678, 2)                # 12,345.69
#   x=commaize(12345.678, 1)                # 12,345.7
#   x=commaize(1234, ".")                   # 1.234
#   x=commaize("1234567,89", -1, ".", ",")  # EU: 1.234.567,89
# 
# The arguments are:
#    
# @param num  The number: int, float or string.
# @param dpl  Decimal places (default: -1 -> do nothing).
# @param sep  Thousands separator (default: ",").
# @param dpt  Decimal point (default: ".").
# @returns a formatted string.
# ================================================================
function commaize($num, $dpl=-1, $sep=",", $dpt=".") {
    $rep = $num;

    # Format to N decimal places.
    if ($dpl >= 0) {
        $fmt = "%.${dpl}f";
        $rep = sprintf($fmt, $num);
    }

    # Split into parts.
    $parts = explode($dpt, $rep);


    # Get the characteristic and mantissa.
    #   123.45
    #   ^  ^^
    #   |  |+--- mantissa
    #   |  +---- decimal point
    #   +------- characteristic
    $characteristic = $parts[0];
    $mantissa = count($parts) == 1 ? "" : $dpt . $parts[1];

    # Handle negative numbers.
    $neg = "";
    if ($characteristic[0] == "-") {
        $neg = "-";
        $characteristic = substr($characteristic, 1);
    }

    # Clean up the mantissa. Remove trailing zeros unless a specific
    # number of decimal places (dpl) was specified.
    #    1.000 > 1
    if (strlen($mantissa) and $dpl < 0) {
        # Make sure that 1.0 --> 1
        $mantissa = rtrim($mantissa, "0");
        $mantissa = rtrim($mantissa, $dpt);
    }

    # Insert the commas moving from left to right.
    $result = "";  # where to put the result
    $offset = strlen($characteristic) % 3;  # offset to start inserting commas
    for($i=0; $i<strlen($characteristic); $i++) {
        if (($i%3) == $offset and $i > 0) {
            $result .= $sep;
        }
        $result .= $characteristic[$i];
    }

    $final = $neg . $result . $mantissa;
    return $final;
}

# ================================================================
# main
# ================================================================
function main() {
    $data = array (1, 12, 123, 1234, 12345, 123456, 1234567, 12345678, 123456789,
                   1.23, 12.34, 123.45, 1234.56, 12345.67, 123456.78, 1234567.89, 12345678.9,
                   1.0,
                   -1, -12, -123, -1234, -12345, -123456, -1234567, -12345678, -123456789,
                   -1.23, -12.34, -123.45, -1234.56, -12345.67, -123456.78, -1234567.89, -12345678.9,
                   -1.0,
                   '123456789',
                   );
    $idx = 1;
    foreach($data as $datum) {
        $result = commaize($datum);
        echo sprintf("%4d %20s  %20s\n", $idx++, "$datum", $result);
    }

    # EU style
    $datum = "12345678,9";
    $result = commaize($datum, -1, ".", ",");
    echo sprintf("%4d %20s  %20s  # EU style\n", $idx++, "$datum", $result);

    # dollarize
    # Round to 2 significant digits.
    $data = array(12345.123, 12345, 12345.1251,);
    foreach($data as $datum) {
        $result = commaize($datum, 2);
        echo sprintf("%4d %20s  %20s  # 2 decimal places\n", $idx++, "$datum", $result);
    }

    # 1 decimal place
    $datum = "1234567.89";
    $result = commaize($datum, 1);
    echo sprintf("%4d %20s  %20s  # 1 decimal place\n", $idx++, "$datum", $result);
}

main();
?>
