#!/usr/bin/env perl
#
# Example that shows how to insert commas into a number.
#
use strict;
use warnings;

&main;

# ================================================================
# main
# ================================================================
sub main
{
    my @data = (1, 12, 123, 1234, 12345, 123456, 1234567, 12345678, 123456789,
		1.23, 12.34, 123.45, 1234.56, 12345.67, 123456.78, 1234567.89, 12345678.9,
		1.0,
		-1, -12, -123, -1234, -12345, -123456, -1234567, -12345678, -123456789,
		-1.23, -12.34, -123.45, -1234.56, -12345.67, -123456.78, -1234567.89, -12345678.9,
		-1.0,
		'123456789',
	);

    my $idx = 1;
    foreach my $datum (@data) {
        my $result = &commaize($datum);
	printf("%4d %20s  %20s\n", $idx++, $datum, $result);
    }

    # EU style
    my $datum = "12345678,9";
    my $result = &commaize($datum, -1, ".", ",");  # EU style
    printf("%4d %20s  %20s  # EU style\n", $idx++, $datum, $result);

    # dollarize
    # Round to 2 significant digits.
    my @ddata = (12345.123, 12345, 12345.1251,);
    for $datum (@ddata) {
	$result = &commaize($datum, 2);
        printf("%4d %20s  %20s  # 2 decimal places\n", $idx++, $datum, $result);
    }

    # 1 decimal place
    $datum = "1234567.89";
    $result = &commaize($datum, 1);
    printf("%4d %20s  %20s  # 1 decimal place\n",$idx++, $datum, $result);
}

# ================================================================
# Insert thousands separator into a number and, optionally, specify
# the number of decimal places.
#    
# Here are some examples that demonstrate how it is used.
#    
#   x=commaize(1234)                        # 1,234
#   x=commaize(-1234)                       # -1,234
#   x=commaize(12345.678)                   # 12,345.678
#   x=commaize(-12345.678)                  # -12,345.678
#   x=commaize(12345.678, 2)                # 12,345.69
#   x=commaize(12345.678, 1)                # 12,345.7
#   x=commaize(1234, -1, ".")               # 1.234
#   x=commaize("1234567,89", -1, ".", ",")  # EU: 1.234.567,89
#    
# The arguments are:
#    
#   @param num  The number: int, float or string.
#   @param dpl  Decimal places (default: -1 -> do nothing).
#   @param sep  Thousands separator (default: ",").
#   @param dpt  Decimal point (default: ".").
#   @returns a formatted string.
# ================================================================
sub commaize
{
    my $num = shift;
    my $dpl = shift;
    my $sep = shift;
    my $dpt = shift;

    $dpl = -1  if( !defined($dpl) );
    $sep = "," if( !defined($sep) );
    $dpt = "." if( !defined($dpt) );

    # Format to N decimal places.
    my $rep = $num;
    if ($dpl >= 0) {
	my $fmt = "%.${dpl}f";
	$rep = sprintf($fmt, $num);
    }

    # Split into parts.
    # Using split is tricky here because the split value is unknown,
    # i work around that using this simple brute force approach.
    my @parts = ();
    my @chars = split(//, $rep);
    my $idx = 0;
    for my $char (@chars) {
	if ($char eq $dpt) {
	    $idx++;
	}
	else {
	    if ($#parts <= $idx) {
		push(@parts, "");
	    }
	    $parts[$idx] .= $char;
	}
    }

    # Get the characteristic and mantissa.
    #   123.45
    #   ^  ^^
    #   |  |+--- mantissa
    #   |  +---- decimal point
    #   +------- characteristic
    my $characteristic = $parts[0];
    my $mantissa = "";
    $mantissa = "$dpt$parts[1]" if $#parts > 1;

    # Handle negative numbers.
    my $neg = "";
    if (substr($characteristic, 0, 1) eq "-") {
	$neg = "-";
	$characteristic = substr($characteristic, 1);
    }

    # Insert the commas moving from left to right.
    my $result = "";  # where to put the result
    my $offset = length($characteristic) % 3;  # offset to start inserting commas
    for(my $i=0; $i<length($characteristic); $i++) {
	if (($i % 3) == $offset && $i) {
	    $result .= $sep;
	}
	$result .= substr($characteristic, $i, 1);
    }

    my $final = $neg . $result . $mantissa;
    return $final;
}

