#!/usr/bin/env python
'''
Example that shows how to insert commas into a number.
'''
import re


def commaize(num, dpl=-1, sep=',', dpt='.'):
    '''
    Insert thousands separator into a number and, optionally, specify
    the number of decimal places.
    
    Here are some examples that demonstrate how it is used.
    
      x=commaize(1234)                            # 1,234
      x=commaize(-1234)                           # -1,234
      x=commaize(12345.678)                       # 12,345.678
      x=commaize(-12345.678)                      # -12,345.678
      x=commaize(12345.678, dpl=2)                # 12,345.69
      x=commaize(12345.678, dpl=1)                # 12,345.7
      x=commaize(1234,sep='.')                    # 1.234
      x=commaize("1234567,89", sep='.', dpt=',')  # EU: 1.234.567,89
    
    The arguments are:
    
    @param num  The number: int, float or string.
    @param dpl  Decimal places (default: -1 -> do nothing).
    @param sep  Thousands separator (default: ",").
    @param dpt  Decimal point (default: ".").
    @returns a formatted string.
    '''
    rep = str(num)  # convert the int or float number to a string
    assert rep.find(sep) < 0  # no separators allowed

    # Format to N decimal places.
    if dpl >= 0:
        fmt = '%.' + str(dpl) + 'f'
        rep = fmt % (float(num))

    # Split into parts.
    parts = rep.split(dpt)  # 123 or 123.45
    assert len(parts) < 3   # multiple dpts not allowed

    # Get the characteristic and mantissa.
    #   123.45
    #   ^  ^^
    #   |  |+--- mantissa
    #   |  +---- decimal point
    #   +------- characteristic
    characteristic = parts[0]
    mantissa = "" if len(parts) == 1 else dpt + parts[1]

    # Handle negative numbers.
    neg = ''
    if characteristic[0] == '-':
        neg = '-'
        characteristic = characteristic[1:]  # strip off the negative

    # Check our assumption that both the characteristic and the mantissa
    # are integers at this point.
    assert re.match(r'^\d+$', characteristic)  # empty strings are not allowed
    if len(mantissa):  # empty strings are allowed
        assert re.match(r'^' + dpt + '\d+$', mantissa)

    # Clean up the mantissa. Remove trailing zeros unless a specific
    # number of decimal places (dpl) was specified.
    #    1.000 > 1
    if len(mantissa) and dpl < 0:
        # Make sure that 1.0 --> 1
        mantissa = mantissa.rstrip('0')
        mantissa = mantissa.rstrip(dpt)

    # Insert the commas moving from left to right.
    result = ''  # where to put the result
    offset = len(characteristic) % 3  # offset to start inserting commas
    for i in range(len(characteristic)):
        if (i%3) == offset and i:
            # if we are at the digit where a separator needs to be
            # pre-pended, prepend the separator.
            result += sep 
        result += characteristic[i]

    final = neg + result + mantissa
    return final


def main():
    '''
    Test the method.
    '''
    idx = [0]
    def incidx():
        idx[0] += 1
        return idx[0]

    data = [1, 12, 123, 1234, 12345, 123456, 1234567, 12345678, 123456789,
            1.23, 12.34, 123.45, 1234.56, 12345.67, 123456.78, 1234567.89, 12345678.9,
            1.0,
            -1, -12, -123, -1234, -12345, -123456, -1234567, -12345678, -123456789,
            -1.23, -12.34, -123.45, -1234.56, -12345.67, -123456.78, -1234567.89, -12345678.9,
            -1.0,
            '123456789',
            ]
    for datum in data:
        result = commaize(datum)
        print('%4d %20s  %20s' % (incidx(), str(datum), result))

    # EU style
    datum = "12345678,9"
    result = commaize(datum, sep='.', dpt=',')  # EU style
    print('%4d %20s  %20s  # EU style' % (incidx(), str(datum), result))

    # dollarize
    # Round to 2 significant digits.
    data = [12345.123, 12345, 12345.1251,]
    for datum in data:
        result = commaize(datum, dpl=2)
        print('%4d %20s  %20s  # 2 decimal places' % (incidx(), str(datum), result))

    # 1 decimal place
    datum = "1234567.89"
    result = commaize(datum, dpl=1)
    print('%4d %20s  %20s  # 1 decimal place' % (incidx(), str(datum), result))


if __name__ == '__main__':
    main()
