#!/usr/bin/env ruby

# ================================================================
# Insert thousands separator into a number and, optionally, specify
# the number of decimal places.
#    
# Here are some examples that demonstrate how it is used.
#    
#   x=commaize(1234)                        # 1,234
#   x=commaize(-1234)                       # -1,234
#   x=commaize(12345.678)                   # 12,345.678
#   x=commaize(-12345.678)                  # -12,345.678
#   x=commaize(12345.678, 2)                # 12,345.69
#   x=commaize(12345.678, 1)                # 12,345.7
#   x=commaize(1234, -1, '.')               # 1.234
#   x=commaize("1234567,89", -1, '.', ',')  # EU: 1.234.567,89
#    
# The arguments are:
#    
#   @param num  The number: int, float or string.
#   @param dpl  Decimal places (default: -1 -> do nothing).
#   @param sep  Thousands separator (default: ",").
#   @param dpt  Decimal point (default: ".").
#   @returns a formatted string.
# ================================================================
def commaize(num, dpl=-1, sep=',', dpt='.')
  snum = num.to_s
  raise "invalid number format" unless snum.count(sep.to_s) == 0

  # Format to N decimal places.
  if (dpl >= 0) then
    fmt = '%.' << dpl.to_i.to_s << 'f'
    snum = sprintf fmt, snum.to_f
  end

  # Split into parts.
  parts = snum.split(dpt)
  raise "invalid number of parts: %d" % (parts.length) unless parts.length < 3

  # Get the characteristic and mantissa.
  #   123.45
  #   ^  ^^
  #   |  |+--- mantissa
  #   |  +---- decimal point
  #   +------- characteristic
  characteristic = parts[0]
  mantissa = parts.length > 1 ? dpt + parts[1] : ""

  # Handle negative numbers.
  neg = ""
  if (characteristic[0] == '-')
    characteristic = characteristic[1..-1]
    neg = "-"
  end

   # Clean up the mantissa. Remove trailing zeros unless a specific
  # number of decimal places (dpl) was specified.
  #    1.000 > 1
  if (mantissa.length > 0 and dpl < 0) then
    # Make sure that 1.0 --> 1
    while (mantissa[-1] == '0')
      mantissa = mantissa.chomp("0")
    end
    mantissa = mantissa.chomp(dpt)
  end

  # Insert the commas moving from left to right.
  result = ""
  j = characteristic.length % 3
  characteristic.split("").each_with_index do |item, i|
    if ((i%3) == j and i > 0) then
      # if we are at the digit where a separator needs to be
      # pre-pended, prepend the separator.
      result << sep
    end
    result << item
  end

  if (mantissa.length) then
    result << mantissa
  end

  return neg + result
end

# ================================================================
# Test method.
# ================================================================
def test()
  data = [1, 12, 123, 1234, 12345, 123456, 1234567, 12345678, 123456789,
          1.23, 12.34, 123.45, 1234.56, 12345.67, 123456.78, 1234567.89, 12345678.9,
          1.0,
          -1, -12, -123, -1234, -12345, -123456, -1234567, -12345678, -123456789,
          -1.23, -12.34, -123.45, -1234.56, -12345.67, -123456.78, -1234567.89, -12345678.9,
          -1.0,
          '123456789',
         ]
  idx = 1
  incidx = lambda do
    idx += 1
    return idx
  end
  data.each do |num|
    result = commaize(num)
    print("%4d %20s  %20s\n" % [incidx.call(), num.to_s, result])
  end

  # EU style
  datum = "12345678,9"
  result = commaize(datum, -1, '.', ',')  # EU style
  print("%4d %20s  %20s  # EU style\n" % [incidx.call(), datum.to_s, result])

  # dollarize
  # Round to 2 significant digits.
  data = [12345.123, 12345, 12345.1251,]
  data.each do |num|
    result = commaize(num, 2)
    print("%4d %20s  %20s  # 2 decimal places\n" % [incidx.call(), num.to_s, result])
  end
  
  # 1 decimal place
  datum = "1234567.89"
  result = commaize(datum, dpl=1)
  print("%4d %20s  %20s  # 1 decimal place\n" % [incidx.call(), datum.to_s, result])

end

if __FILE__ == $0
  test()
end
