#!/bin/bash

#
# Insert thousands separator into a number and, optionally, specify
# the number of decimal places.
#
# Here are some examples that demonstrate how it is used.
#
#    x=$(commaize 1234)                      # 1,234
#    x=$(commaize -1234)                     # -1,234
#    x=$(commaize 12345.678)                 # 12,345.678
#    x=$(commaize -12345.678)                # -12,345.678
#    x=$(commaize 12345.678 2)               # 12,345.69
#    x=$(commaize 12345.678 1)               # 12,345.7
#    x=$(commaize 1234 -1 ".")               # 1.234
#    x=$(commaize "1234567,89" -1 "." ",")   # EU format: 1.234.567,89
#
# The arguments are:
#
#   $1 == number
#   $2 == decimal places (default: -1 -> do nothing)
#   $3 == thousands separator (default: ",")
#   $4 == decimal point (default: ".")
#
function commaize() {
    local num="$1"
    local dpl=-1
    local sep=","
    local dpt="."
    if [[ "$2" != "" ]] ; then dpl=$2 ; fi
    if [[ "$3" != "" ]] ; then sep="$3" ; fi
    if [[ "$4" != "" ]] ; then dpt="$4" ; fi

    if (( $dpl >= 0 )) ; then
        fmt="%.${dpl}f"
        num=$(printf "$fmt" $num)
    fi

    local neg=""
    if [[ "${num:0:1}" == "-" ]] ; then
        neg="-"
        # Strip off the leading minus sign.
        numl=$((${#num} - 1))
        num=${num:1:$numl}
    fi

    local parts=($(echo "$num" | tr "$dpt" "\n"))
    local nparts=${#parts[@]}
    if (( $nparts > 2 )) ; then
        echo "ERROR: invalid string: $num"
        exit 1
    fi

    # Insert the separators.
    characteristic=${parts[0]}
    result=''
    j=$(( ${#characteristic} % 3 ))
    for (( i=0; i<${#characteristic}; i++ )); do
        k=$(( $i % 3))
        if (( $k == $j )) && (( $i > 0 )) ; then
            result="$result$sep"
        fi
        digit=${characteristic:$i:1}
        result="$result$digit"
    done

    # There was a decimal point, append
    # the mantissa.
    if (( $nparts == 2 )) ; then
        mantissa="${parts[1]}"
        if (( $dpl < 0 )) ; then
            # Strip off trailing zeros if dpl was not not specified.
            # Convert 1.0 --> 1
            mantissa=$(echo "$mantissa" | sed -e 's/0$//g')
        fi
        if (( ${#mantissa} > 0 )) ; then
            result="$result$dpt$mantissa"
        fi
    fi
    echo "$neg$result"
}

# Main
idx=0
data=(1 12 123 1234 12345 123456 1234567 12345678 123456789
    1.23 12.34 123.45 1234.56 12345.67 123456.78 1234567.89 12345678.9
    1.0
    -1 -12 -123 -1234 -12345 -123456 -1234567 -12345678 -123456789
    -1.23 -12.34 -123.45 -1234.56 -12345.67 -123456.78 -1234567.89 -12345678.9
    -1.0
    '123456789')
for datum in ${data[@]}; do
    result=$(commaize $datum)
    idx=$(($idx + 1))
    printf "%4d %20s  %20s\n" $idx $datum $result
done

datum="12345678,9"
result=$(commaize $datum -1 '.' ',')
idx=$(($idx + 1))
printf "%4d %20s  %20s  # EU style\n" $idx $datum $result

data=(12345.123 12345 12345.1251)
for datum in ${data[@]}; do
    result=$(commaize $datum 2)
    idx=$(($idx + 1))
    printf "%4d %20s  %20s  # 2 decimal places\n" $idx $datum $result
done

datum=1234567.89
result=$(commaize $datum 1)
idx=$(($idx + 1))
printf "%4d %20s  %20s  # 1 decimal place\n" $idx $datum $result
