#!/usr/bin/env tclsh
#
# Example that shows how to insert commas into a number.
#

# ================================================================
# Insert thousands separator into a number and, optionally, specify
# the number of decimal places.
#    
# Here are some examples that demonstrate how it is used.
#    
#   set x [commaize 1234]                  # 1,234
#   set x [commaize -1234]                 # -1,234
#   set x [commaize 12345.678]             # 12,345.678
#   set x [commaize -12345.678]            # -12,345.678
#   set x [commaize 12345.678 2]           # 12,345.69
#   set x [commaize 12345.678 1]           # 12,345.7
#   set x [commaize 1234 -1 "."]           # 1.234
#   set x [commaize "1234567,89" "." ","]  # EU: 1.234.567,89
# 
# The arguments are:
#    
# @param num  The number: int, float or string.
# @param dpl  Decimal places (default: -1 -> do nothing).
# @param sep  Thousands separator (default: ",").
# @param dpt  Decimal point (default: ".").
# @returns a formatted string.
# ================================================================
proc commaize {num {dpl -1} {sep ","} {dpt "."}} {
    set rep $num

    # Format to N decimal places.
    if {$dpl >= 0} {
        set fmt "%.${dpl}f"
        set rep [format $fmt $num]
    }

    # Split into parts.
    set parts [split $rep $dpt]
    set size [llength $parts]

    # Get the characteristic and mantissa.
    #   123.45
    #   ^  ^^
    #   |  |+--- mantissa
    #   |  +---- decimal point
    #   +------- characteristic
    set characteristic [lindex $parts 0]
    if {$size == 1} {
        set mantissa ""
    } else {
        set num [lindex $parts 1]
        set mantissa "${dpt}${num}"
    }

    # Trim off trailing zeros.
    if {$dpl < 0} {
        set mantissa [string trimright $mantissa "0"]
        set mantissa [string trimright $mantissa $dpt]
    }

    # Handle negative numbers.
    set neg [string index $characteristic 0]
    if {[string compare $neg "-"] == 0} {
        # strip off the negative
        set characteristic [string range $characteristic 1 [string length $characteristic]]
    } else {
        set neg ""
    }

    # Insert the commas moving from left to right.
    set result ""
    set offset [expr [string length $characteristic] % 3]
    for {set i 0} {$i < [string length $characteristic]} {incr i} {
        if {[expr $i>0] && [expr ($i%3)] == $offset} {
            append result $sep
        }
        append result [string index $characteristic $i]
    }

    set final "${neg}${result}${mantissa}"
    return $final
}

# ================================================================
# main
# ================================================================
proc main {} {
    set data [list 1 12 123 1234 12345 123456 1234567 12345678 123456789 \
            1.23 12.34 123.45 1234.56 12345.67 123456.78 1234567.89 12345678.9 \
            1.0 \
            -1 -12 -123 -1234 -12345 -123456 -1234567 -12345678 -123456789 \
            -1.23 -12.34 -123.45 -1234.56 -12345.67 -123456.78 -1234567.89 -12345678.9 \
            -1.0 \
            "123456789" \
            ]

    set idx 0
    foreach datum $data {
        incr idx
        set result [commaize $datum]
        set f [format "%4d %20s %20s" $idx $datum $result]
        puts $f
    }

    # EU style
    set datum "12345678,9"
    incr idx
    set result [commaize $datum -1 "." ","]
    set f [format "%4d %20s %20s" $idx $datum $result]
    puts $f

    # dollarize
    # Round to 2 significant digits
    set data [list 12345.123 12345 12345.1251]
    foreach datum $data {
        incr idx
        set result [commaize $datum 2]
        set f [format "%4d %20s %20s" $idx $datum $result]
        puts $f
    }

    # 1 decimal place
    set datum "1234567.89"
    incr idx
    set result [commaize $datum 1]
    set f [format "%4d %20s %20s" $idx $datum $result]
    puts $f
}

main
